import { useCallback, useContext, useEffect, useState, useMemo } from "react";
import {
  Button,
  TextField,
  Typography,
  CircularProgress,
  Autocomplete,
  Alert,
  Stack
} from "@mui/material";
import { apiContext } from "../../../api/DeployApi";
import {
  useAPIMethod,
  Dialog,
  GitRefAutocomplete
} from "@ess-ics/ce-ui-common";

export function IOCDeployDialog({
  open,
  setOpen,
  iocId,
  submitCallback,
  hasActiveDeployment,
  deployIocFormDefaults = {},
  error,
  resetError,
  buttonDisabled,
  setButtonDisabled
}) {
  const client = useContext(apiContext);

  const {
    value: hosts,
    wrapper: getHosts,
    loading: loadingHosts
  } = useAPIMethod({
    fcn: client.apis.Hosts.listHosts,
    call: false
  });

  const {
    value: tagOrCommitId,
    wrapper: callGetTagOrCommitId,
    loading: loadingTagsAndCommitIds,
    error: tagOrCommitIdError
  } = useAPIMethod({
    fcn: client.apis.Git.listTagsAndCommitIds,
    call: false,
    init: []
  });

  const getTagOrCommitId = useCallback(
    (gitProjectId, reference, includeAllReference, searchMethod) => {
      callGetTagOrCommitId({
        project_id: gitProjectId,
        reference: reference,
        include_all_reference: includeAllReference,
        search_method: searchMethod
      });
    },
    [callGetTagOrCommitId]
  );

  const initHost = useMemo(
    () => ({
      fqdn: deployIocFormDefaults.netBoxHost?.fqdn,
      hostId: deployIocFormDefaults.netBoxHost?.hostId
    }),
    [deployIocFormDefaults.netBoxHost]
  );

  const [host, setHost] = useState(initHost);
  const [query, setQuery] = useState("");
  const [gitRepo, setGitRepo] = useState(deployIocFormDefaults.git || "");
  const [revision, setRevision] = useState(deployIocFormDefaults);
  const gitProjectId = deployIocFormDefaults.gitProjectId;

  const handleClose = () => {
    setOpen(false);
  };

  useEffect(() => {
    if (query && query.length > 2) {
      getHosts({ text: `${query}` });
    }
  }, [query, getHosts]);

  useEffect(() => {
    getTagOrCommitId(gitProjectId, "", false, "CONTAINS");
  }, [gitProjectId, getTagOrCommitId]);

  const onSubmit = (event) => {
    event.preventDefault();
    setButtonDisabled(true);
    const { git: gitText } = event.currentTarget.elements;
    const git = gitText.value;

    submitCallback(
      {
        ioc_id: iocId
      },
      {
        requestBody: {
          sourceUrl: git,
          sourceVersion: revision.reference,
          hostId: host.hostId,
          type: "DEPLOY"
        }
      }
    );
  };

  return (
    <Dialog
      open={open}
      onClose={handleClose}
      title={
        <Typography
          variant="h2"
          marginY={1}
        >
          {hasActiveDeployment ? "Deploy revision" : "Deploy"}
        </Typography>
      }
      content={
        <Stack
          component="form"
          onSubmit={onSubmit}
          gap={1}
        >
          <TextField
            autoComplete="off"
            id="git"
            label="Git repository"
            variant="standard"
            defaultValue={deployIocFormDefaults.git || ""}
            fullWidth
            onChange={(event) => {
              setGitRepo(event.target.value);
              resetError();
            }}
            disabled
            required
          />
          <GitRefAutocomplete
            id="version"
            label="Revision"
            options={tagOrCommitId}
            loading={loadingTagsAndCommitIds}
            disabled={!gitRepo || gitRepo.trim() === ""}
            autocompleteProps={{
              autoSelect: false,
              defaultValue: tagOrCommitId.find(
                (tagOrCommit) =>
                  tagOrCommit.reference === deployIocFormDefaults.reference
              )
            }}
            onGitQueryValueSelect={(_, value) => {
              setRevision(value);
              resetError();
            }}
            textFieldProps={{
              helperText: tagOrCommitIdError
                ? `Error: ${tagOrCommitId?.message}`
                : ""
            }}
          />

          <Autocomplete
            autoHighlight
            id="host"
            options={query ? hosts?.netBoxHosts ?? [] : []}
            loading={loadingHosts}
            clearOnBlur={false}
            value={host}
            getOptionLabel={(option) => {
              return option?.fqdn ?? "";
            }}
            renderInput={(params) => (
              <TextField
                {...params}
                label="Host"
                variant="outlined"
                required
                InputProps={{
                  ...params.InputProps,
                  endAdornment: (
                    <>
                      {loadingHosts ? (
                        <CircularProgress
                          color="inherit"
                          size={20}
                        />
                      ) : null}
                      {params.InputProps.endAdornment}
                    </>
                  )
                }}
              />
            )}
            onChange={(_event, value) => {
              setHost(value);
              resetError();
            }}
            onInputChange={(_event, value) => setQuery(value)}
            disabled={hasActiveDeployment}
            autoSelect
            filterOptions={(options) => options}
          />

          {hasActiveDeployment ? (
            <Typography variant="body2">
              Hint: First undeploy IOC if you want to move to another host.
            </Typography>
          ) : (
            <></>
          )}

          {error ? <Alert severity="error">{error}</Alert> : <></>}
          <Stack
            flexDirection="row"
            justifyContent="flex-end"
            gap={2}
            marginTop={1}
          >
            <Button
              onClick={handleClose}
              color="primary"
            >
              Cancel
            </Button>
            <Button
              color="primary"
              variant="contained"
              type="submit"
              disabled={!host || !revision || buttonDisabled}
            >
              Deploy
            </Button>
          </Stack>
        </Stack>
      }
    />
  );
}
