import { useState, useEffect, useCallback, useMemo } from "react";
import { useLazyListIocsQuery, ListIocsApiArg } from "../../store/deployApi";
import {
  useGlobalAppBarContext,
  RootPaper,
  usePagination
} from "@ess-ics/ce-ui-common";
import {
  applicationTitle,
  initRequestParams
} from "../../components/common/Helper";
import useUrlState from "@ahooksjs/use-url-state";
import {
  serialize,
  deserialize
} from "../../components/common/URLState/URLState";
import { GlobalAppBarContext, OnPageParams } from "../../types/common";
import { SearchBar } from "../../components/common/SearchBar/SearchBar";
import { ApiAlertError } from "../../components/common/Alerts/ApiAlertError";
import { Container, Grid, Tabs, Tab } from "@mui/material";
import IOCTable from "../../components/IOC/IOCTable";

const ROWS_PER_PAGE = [20, 50];

export const IOCListView = () => {
  const [deploymentStatus, setDeploymentStatus] =
    useState<ListIocsApiArg["deploymentStatus"]>("ALL");
  const [listIocs, { data: iocs, isFetching, error }] = useLazyListIocsQuery();
  const [urlState, setUrlState] = useUrlState(
    {
      tab: "0",
      rows: "20",
      page: "0",
      query: ""
    },
    { navigateMode: "replace" }
  );
  const { setTitle }: GlobalAppBarContext = useGlobalAppBarContext();

  const handleTabChange = useCallback(
    (tab: number) => {
      setUrlState((s) =>
        serialize(s.tab) === serialize(tab)
          ? { tab: serialize(tab) }
          : { tab: serialize(tab), page: "0" }
      );
      changeTab(tab);
    },
    [setUrlState]
  );

  const changeTab = (tab: number) => {
    if (tab === 0) {
      setDeploymentStatus("ALL");
    } else if (tab === 1) {
      setDeploymentStatus("DEPLOYED");
    } else if (tab === 2) {
      setDeploymentStatus("NOT_DEPLOYED");
    }
  };

  const setUrlPagination = useCallback(
    ({ rows, page }: OnPageParams) => {
      setUrlState({
        rows: serialize(rows),
        page: serialize(page)
      });
    },
    [setUrlState]
  );

  const urlPagination = useMemo(() => {
    return {
      rows: deserialize(urlState.rows),
      page: deserialize(urlState.page)
    };
  }, [urlState.rows, urlState.page]);

  const { pagination, setPagination } = usePagination({
    rowsPerPageOptions: ROWS_PER_PAGE,
    initLimit: urlPagination.rows ?? ROWS_PER_PAGE[0],
    initPage: urlPagination.page ?? 0
  });

  // Invoked by Table on change to pagination
  const onPage = (params: OnPageParams) => {
    setPagination(params);
  };

  // Callback for searchbar, called whenever user updates search
  const setSearch = useCallback(
    (query: string) => {
      setUrlState({ query: serialize(query) });
    },
    [setUrlState]
  );

  useEffect(() => setTitle(applicationTitle("IOCs")), [setTitle]);

  useEffect(() => {
    if (urlState.tab) {
      changeTab(deserialize(urlState.tab));
    }
  }, [urlState.tab]);

  // update pagination whenever search result total pages change
  useEffect(() => {
    setPagination({ totalCount: iocs?.totalCount ?? 0 });
  }, [setPagination, iocs?.totalCount]);

  // whenever url state changes, update pagination
  useEffect(() => {
    setPagination({ ...urlPagination });
  }, [setPagination, urlPagination]);

  // whenever table pagination internally changes (user clicks next page etc)
  // update the row params
  useEffect(() => {
    setUrlPagination(pagination);
  }, [pagination, setUrlPagination]);

  useEffect(() => {
    const requestParams = Object.assign(
      {},
      initRequestParams(pagination, deserialize(urlState.query)),
      { deploymentStatus: deploymentStatus }
    );

    listIocs(requestParams);
  }, [listIocs, urlPagination, deploymentStatus, urlState.query, pagination]);

  return (
    <RootPaper>
      <Grid
        container
        spacing={1}
      >
        <Grid
          container
          spacing={1}
          component={Container}
          justifyContent="space-between"
          alignItems="center"
          style={{ display: "flex" }}
        >
          <Grid item>
            <Tabs
              value={deserialize(urlState.tab)}
              onChange={(event, tab) => handleTabChange(tab)}
            >
              <Tab label="All" />
              <Tab label="Only Deployed" />
              <Tab label="Only Not Deployed" />
            </Tabs>
          </Grid>
        </Grid>
        <Grid
          item
          xs={12}
          md={12}
        >
          <SearchBar
            search={setSearch}
            query={deserialize(urlState.query)}
            loading={isFetching || !iocs}
            placeholder="Search"
          >
            {error ? (
              <ApiAlertError error={error} />
            ) : (
              <IOCTable
                iocs={iocs?.iocList}
                loading={isFetching || !iocs}
                rowType="explore"
                pagination={pagination}
                onPage={onPage}
              />
            )}
          </SearchBar>
        </Grid>
      </Grid>
    </RootPaper>
  );
};
